import globalPluginHandler
import errors
import re
from sys import path
import addonHandler
import codecs
import api
from speech import cancelSpeech
import os
import gui
import wx
import ui
import scriptHandler
addonHandler.initTranslation()

_basePath = os.path.dirname(os.path.abspath(__file__))
_savePath = os.path.join(_basePath, "files")
virtualWindowViewer = None
from reader import Rtf15Reader
from writer import PlaintextWriter

class GlobalPlugin(globalPluginHandler.GlobalPlugin):
	lastFindText=""
	def __init__(self):
		super(globalPluginHandler.GlobalPlugin, self).__init__()
		self.menu = gui.mainFrame.sysTrayIcon.menu
		self.fileReaderMenu = wx.Menu()
		self.menu.AppendSubMenu(self.fileReaderMenu, _("&File Reader"), "File reader")
		self.loadFiles= self.fileReaderMenu.Append(wx.ID_ANY, _("&Load list of files... alt+NVDA+l"), _("Load a list of files for reading"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onLoadFiles, self.loadFiles)
		self.loadAll= self.fileReaderMenu.Append(wx.ID_ANY, _("Load &all files alt+NVDA+a"), _("Load all files for reading"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onLoadAll, self.loadAll)
		self.openFolder= self.fileReaderMenu.Append(wx.ID_ANY, _("&Go to files folder... alt+NVDA+i"), _("open the files folder in explorer"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onFolder, self.openFolder)
		self.goToTop = self.fileReaderMenu.Append(wx.ID_ANY, _("Go to &top of file alt+NVDA+y"), _("Go to the top of the file"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onTop, self.goToTop)
		self.goToBottom = self.fileReaderMenu.Append(wx.ID_ANY, _("Go to &bottom of file alt+NVDA+n"), _("Go to the bottom of the file"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onBottom, self.goToBottom)
		self.copyLine = self.fileReaderMenu.Append(wx.ID_ANY, _("&Copy line to clipboard... alt+NVDA+c"), _("Copy the current line to the clipboard"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onCopyLine, self.copyLine)
		self.findText = self.fileReaderMenu.Append(wx.ID_ANY, _("&Find Text... alt+NVDA+f"), _("Find text on current file"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onFind, self.findText)
		self.findNextText = self.fileReaderMenu.Append(wx.ID_ANY, _("Find &Next alt+NVDA+g"), _("Find next instance of text"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onFindNext, self.findNextText)
		self.showFile = self.fileReaderMenu.Append(wx.ID_ANY, _("&Show file on screen... alt+NVDA+s"), _("Show file on screen"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onShowFile, self.showFile)


	def terminate(self):
		try:
			gui.initialize()
		except wx.PyDeadObjectError:
			pass
	currentFile=0
	count=0
	throughFile=False
	files=[]

	def onCopyLine(self, evt):
		scriptHandler.executeScript(self.script_copyLine, None)
	def onBottom(self, evt):
		scriptHandler.executeScript(self.script_goBottom, None)
	def onFolder(self, evt):
		scriptHandler.executeScript(self.script_openFilesFolder, None)
	def onLoadAll(self, evt):
		wx.CallLater(100,self.script_loadAll, None)

	def onTop(self, evt):
		scriptHandler.executeScript(self.script_goTop, None)

	def onLoadFiles(self, evt):
		scriptHandler.executeScript(self.script_loadFiles, None)
	def onShowFile(self, evt):
		scriptHandler.executeScript(self.script_showFile, None)

	def onFind(self, evt):
		scriptHandler.executeScript(self.script_findText, None)
	def onFindNext(self, evt):
		wx.CallLater(100,self.script_findNextInstance, None)

	def findTextDialog(self):
		d = wx.PasswordEntryDialog(gui.mainFrame, _("Enter the text you wish to find in {0}:").format(self.files[self.currentFile]["name"]), _("Find in {0}").format(self.files[self.currentFile]["name"]))
		def callback(result):
			if result == wx.ID_OK:
				self.doFindText(d.GetValue())
		gui.runScriptModalDialog(d, callback)

	def doFindText(self, text):
		found=False
		if not text:
			return
		text=text.lower()
		self.lastFindText=text
		self.throughFile=False
		pos=self.files[self.currentFile]["pos"]
		if pos==-1: pos=0
		count=pos
		while count<len(self.files[self.currentFile]["lines"])-1:
			count+=1
			if not text in self.files[self.currentFile]["lines"][count].lower(): 
				continue
			else:
				found=True
				self.files[self.currentFile]["pos"]=count
				break
			count+=1
		if not found:
			cancelSpeech()
			ui.message(_("Text not found"))
			return
		cancelSpeech()
		ui.message("%s"%self.files[self.currentFile]["lines"][count])
		ui.message(_("Line {0} of {1}.").format(self.files[self.currentFile]["pos"]+1,len(self.files[self.currentFile]["lines"])))
		return
	def script_findNextInstance(self,gesture):
		if self.files==[]:
			ui.message(_("There are no files loaded."))
			return
		self.doFindText(self.lastFindText)
#s1
	def loadFilesDialog(self):
		d = wx.PasswordEntryDialog(gui.mainFrame, _("List of files to load, separated by commas and no extension:"), _("Files"))
		def callback(result):
			if result == wx.ID_OK:
				wx.CallLater(200,self.doLoadFiles(d.GetValue()))
		gui.runScriptModalDialog(d, callback)

	def doLoadFiles(self, text):
		if not text:
			return
		self.files=[]
		self.currentFile=0
		text=text.replace(" ","")
		toLoad=text.split(",")
		errors=""
		for each in toLoad:
			try:
				temp={}
				temp["name"]=each
				if not ".rtf" in each:
					txt=codecs.open(self.getFile(each),encoding="iso8859-1")
					temp["lines"]=txt.readlines()
				else:
					doc = Rtf15Reader.read(open(self.getFile(each), "rb"))
					txt=codecs.decode(PlaintextWriter.write(doc).read(),"utf-8")
					temp["lines"]=txt.split("\n")
				temp["pos"]=-1
				self.files.append(temp)
			except IOError:
				errors+=each+"\n"
				pass
			temp={}
		cancelSpeech()
		ui.message(_("files loaded successfully. Use alt NVDA + left and right arrows to select a file, and up and down to begin reading"))
		if not errors=="":
			string=_("Unable to open %s")%(errors)
			wx.CallAfter(gui.messageBox, string, _("Errors"), wx.OK|wx.ICON_ERROR)
	def script_loadFiles(self, gesture):
		wx.CallLater(100,self.loadFilesDialog())
	def script_copyLine(self, gesture):
		if self.files==[]:
			ui.message(_("There are no files loaded."))
			return
		api.copyToClip(self.files[self.currentFile]["lines"][self.files[self.currentFile]["pos"]])
		ui.message(_("Coppied to clipboard"))
	def script_nextFile(self,gesture):
		if self.files==[]:
			ui.message(_("There are no files loaded."))
			return
		maxFiles=len(self.files)-1
		self.currentFile+=1
		if self.currentFile>maxFiles:
			self.currentFile=0
			ui.message(_("first"))
		ui.message("%s"%self.files[self.currentFile]["name"])
		if self.files[self.currentFile]["pos"]==-1:
			ui.message(_("Use alt NVDA down arrow to begin reading."))
			ui.message(_("This file is {0} lines long.").format(len(self.files[self.currentFile]["lines"])))
		else:
			ui.message(_("Line {0} of {1}.").format(self.files[self.currentFile]["pos"]+1,len(self.files[self.currentFile]["lines"])))
	def script_previousFile(self,gesture):
		if self.files==[]:
			ui.message(_("There are no files loaded."))
			return
		maxFiles=len(self.files)
		self.currentFile-=1
		if self.currentFile<0:
			self.currentFile=maxFiles-1
			ui.message(_("last"))
		ui.message("%s"%self.files[self.currentFile]["name"])
		if self.files[self.currentFile]["pos"]==-1:
			ui.message(_("Use alt NVDA down arrow to begin reading."))
			ui.message(_("This file is {0} lines long.").format(len(self.files[self.currentFile]["lines"])))
		else:
			ui.message(_("Line {0} of {1}.").format(self.files[self.currentFile]["pos"]+1,len(self.files[self.currentFile]["lines"])))

	def script_openFilesFolder(self, gesture):
		import sys
		import config
		import subprocess
		try:
			subprocess.Popen("explorer "+_savePath)
		except WindowsError:
			wx.CallAfter(gui.messageBox, _("Cannot open files path "+_savePath), _("Error"), wx.OK|wx.ICON_ERROR)
	def script_showFile(self,gesture):
		ui.message("ok")
	def script_moveDown(self,gesture):
		if self.files==[]:
			ui.message(_("There are no files loaded."))
			return
		self.files[self.currentFile]["pos"]=		self.files[self.currentFile]["pos"]+1
		if self.files[self.currentFile]["pos"]>len(self.files[self.currentFile]["lines"])-1:
			ui.message(_("bottom"))
			self.files[self.currentFile]["pos"]=len(self.files[self.currentFile]["lines"])-1
		ui.message(self.files[self.currentFile]["lines"][self.files[self.currentFile]["pos"]])

	def script_findText(self,gesture):
		if self.files==[]:
			ui.message(_("There are no files loaded."))
			return
		wx.CallLater(100,self.findTextDialog())

	def script_moveUp(self,gesture):
		if self.files==[]:
			ui.message(_("There are no files loaded."))
			return
		self.files[self.currentFile]["pos"]=		self.files[self.currentFile]["pos"]-1
		if self.files[self.currentFile]["pos"]<0:
			ui.message(_("top"))
			self.files[self.currentFile]["pos"]=0
		ui.message(self.files[self.currentFile]["lines"][self.files[self.currentFile]["pos"]])


#s2
	def standarFileName(self, fileName):
		notAllowed = re.compile("\?|:|\*|\t|<|>|\"|\/|\\||") # Invalid characters
		allowed = re.sub(notAllowed, "", unicode(fileName))
		return allowed

	def getFile(self,name):
		file = name.rsplit(" - ", 1)[0]
		file = file.split("\\")[-1]
		file = self.standarFileName(file)
		if not ".rtf" in file and not ".txt" in file: file = file+".txt"

		path = os.path.join(_savePath, file)
		abspath=os.path.abspath(path)
		return abspath
	def getLastChanges(self):
		f = open(self.getFile(), "r")
		self.lastChanges =f.read()
		f.close()
	def script_goBottom(self,gesture):
		if self.files==[]:
			ui.message(_("There are no files loaded."))
			return
		self.files[self.currentFile]["pos"]=len(self.files[self.currentFile]["lines"])-1
		cancelSpeech()
		ui.message(_("Go to bottom"))
		ui.message(self.files[self.currentFile]["lines"][self.files[self.currentFile]["pos"]])
		ui.message(_("Line {0} of {1}.").format(self.files[self.currentFile]["pos"]+1,len(self.files[self.currentFile]["lines"])))
	def script_goTop(self,gesture):
		if self.files==[]:
			ui.message(_("There are no files loaded."))
			return
		self.files[self.currentFile]["pos"]=0
		cancelSpeech()
		ui.message(_("Go to top"))
		ui.message(self.files[self.currentFile]["lines"][self.files[self.currentFile]["pos"]])
		ui.message(_("Line {0} of {1}.").format(self.files[self.currentFile]["pos"]+1,len(self.files[self.currentFile]["lines"])))
	def script_loadAll(self,gesture):
		flist=os.listdir(_savePath)
		errors=""
		self.files=[]
		self.currentFile=0
		for each in flist:
			try:
				temp={}
				temp["name"]=each
				if not ".rtf" in each:
					txt=codecs.open(self.getFile(each),encoding="iso8859-1")
					temp["lines"]=txt.readlines()
				else:
					doc = Rtf15Reader.read(open(self.getFile(each), "rb"))
					txt=codecs.decode(PlaintextWriter.write(doc).read(),"utf-8")
					temp["lines"]=txt.split("\n")
				temp["pos"]=-1
				self.files.append(temp)
			except IOError:
				errors+=each+"\n"
				pass
			temp={}
		cancelSpeech()
		ui.message(_("files loaded successfully. Use alt NVDA + left and right arrows to select a file, and up and down to begin reading"))
		if not errors=="":
			string=_("Unable to open %s")%(errors)
			wx.CallAfter(gui.messageBox, string, _("Errors"), wx.OK|wx.ICON_ERROR)

	__gestures = {
"kb:alt+NVDA+upArrow": "moveUp",
"kb:alt+NVDA+downArrow": "moveDown",
"kb:alt+NVDA+rightArrow": "nextFile",
"kb:alt+NVDA+leftArrow": "previousFile",
"kb:alt+NVDA+s": "showFile",
"kb:alt+NVDA+f": "findText",
"kb:alt+NVDA+c": "copyLine",
"kb:alt+NVDA+l": "loadFiles",
"kb:alt+NVDA+g":"findNextInstance",
"kb:alt+NVDA+n":"goBottom",
"kb:alt+NVDA+y":"goTop",
"kb:alt+NVDA+i":"openFilesFolder",
"kb:alt+NVDA+a":"loadAll",
}